/*
 * diagonal.hh
 *
 *  Created on: May 7, 2010
 *      Author: mauro
 */

#ifndef MLN_MORPHO_ATTRIBUTE_DIAGONAL_HH
# define MLN_MORPHO_ATTRIBUTE_DIAGONAL_HH

/// \file
///
/// Define an accumulator that computes the length of the diagonal of the bounding box of a
/// component.

# include <mln/core/concept/accumulator.hh>
# include <mln/accu/internal/base.hh>
//# include <mln/morpho/attribute/volume.hh>
//# include <mln/morpho/attribute/height.hh>
# include <mln/accu/shape/bbox.hh>


namespace mln{

// Forward declaration.
namespace morpho {
namespace attribute {
template <typename I>
class diagonal;
}
}


// Traits.

namespace trait{

template <typename I>
struct accumulator_< morpho::attribute::diagonal<I> >
{
	typedef accumulator::has_untake::no    	has_untake;
	typedef accumulator::has_set_value::no 	has_set_value;
	typedef accumulator::has_stop::no      	has_stop;
	typedef accumulator::when_pix::use_p	when_pix;		// Related to the shape of the components
};

} // end of namespace mln::trait


namespace morpho
{

namespace attribute
{

/// Diagonal accumulator class.
///
/// The parameter \p I is the image type on which the accumulator
/// of pixels is built.
template <typename I>
struct diagonal
: public mln::accu::internal::base< double, diagonal<I> >
{
	typedef mln_psite(I) argument;

	diagonal();

	/// Manipulators.
	/// \{
	void init();

	void take(const mln_psite(I)& p);
	void take(const diagonal<I>& other);

	void take_as_init_(const mln_psite(I)& p);
	/// \}

	/// Get the value of the accumulator.
	double to_result() const;

	/// Check whether this accu is able to return a result.
	/// Always true here.
	bool is_valid() const;

//	/// Give the area of the component.
//	unsigned area() const;

//	/// Give the height.
//	unsigned height() const;
//
//	/// Give the volume of the component.
//	unsigned volume() const;

	/// Give the bounding box of the component.
	box<mln_psite(I)> bbox() const;

protected:
	typename mln::accu::shape::bbox<mln_psite(I)> bbox_;
	/// The height of the component.
//	typename mln::morpho::attribute::height<I> height_;
//	/// The volume of the component.
//	typename mln::morpho::attribute::volume<I> volume_;
};



# ifndef MLN_INCLUDE_ONLY

template <typename I>
inline
diagonal<I>::diagonal()
{
	init();
}

template <typename I>
inline
void
diagonal<I>::init()
{
	bbox_.init();
}

template <typename I>
inline
void
diagonal<I>::take(const mln_psite(I)& p)
{
	if (! is_valid())
	{
		take_as_init_(p);
		return;
	}

	bbox_.take(p);
//	volume_.take(v);
//	height_.take(v);
}

template <typename I>
inline
void
diagonal<I>::take(const diagonal<I>& other)
{
	mln_invariant(is_valid());
	bbox_.take(other.bbox_);
//	volume_.take(other.volume_);
//	height_.take(other.height_);
}

template <typename I>
inline
void
diagonal<I>::take_as_init_(const mln_psite(I)& p)
{
	bbox_.take_as_init_(p);
//	volume_.take_as_init_(v);
//	height_.take_as_init_(v);
}

template <typename I>
inline
double
diagonal<I>::to_result() const
{
	box<mln_psite(I)> _bbox_result = bbox_.to_result();
	mln_psite(I) pmax_ = _bbox_result.pmax();
	mln_psite(I) pmin_ = _bbox_result.pmin();
	std::vector<double> tmp;
	double d = 0;
	for(unsigned i = 0; i < _bbox_result.dim; ++i){
		tmp.push_back((double)(pmax_[i] - pmin_[i])+1);
//		std::cout << " pushed back: " << tmp.at(i) << std::endl;
	}

	//   for(tmp.begin(); tmp.end(); tmp++)
	for (unsigned i=0; i<tmp.size(); i++)
		d += tmp.at(i) * tmp.at(i);
////	mln_postcondition(d >= 0);
//	std::cout << "sqrt(d): " << sqrt(d) << std::endl;
	return sqrt(d);
}

template <typename I>
inline
box<mln_psite(I)>
diagonal<I>::bbox() const
{
	return bbox_.to_result();
}

//template <typename I>
//inline
//unsigned
//diagonal<I>::area() const
//{
//	return volume_.area();
//}
//
//template <typename I>
//inline
//unsigned
//sharpness<I>::volume() const
//{
//	return volume_.to_result();
//}
//
//template <typename I>
//inline
//unsigned
//diagonal<I>::height() const
//{
//	return height_.to_result();
//}

template <typename I>
inline
bool
diagonal<I>::is_valid() const
{
	return bbox_.is_valid();
}

# endif // ! MLN_INCLUDE_ONLY

} // end of namespace mln::morpho::attribute

} // end of namespace mln::morpho

} // end of namespace mln


#endif // ! MLN_MORPHO_ATTRIBUTE_DIAGONAL_HH

