/*
 * inertia.hh
 *
 *  Created on: May 12, 2010
 *      Author: mauro
 */


#ifndef MLN_MORPHO_ATTRIBUTE_INERTIA_HH
#define MLN_MORPHO_ATTRIBUTE_INERTIA_HH

#include <mln/accu/shape/inertia2d.hh>

/// \file
///
/// Define an accumulator that computes the moment of inertia of the components.
/// The variance is normalized by N, the number of sites considered (instead the unbiased estimation normalizes by N-1).

namespace mln{

// Forward declaration.
namespace morpho {
namespace attribute {
template <typename P>
class inertia;
}
}

// Traits.

namespace trait{

template <typename I>
struct accumulator_< morpho::attribute::inertia<I> >
{
	typedef accumulator::has_untake::no    	has_untake;
	typedef accumulator::has_set_value::no 	has_set_value;
	typedef accumulator::has_stop::no      	has_stop;
	typedef accumulator::when_pix::use_p	when_pix;		// Related to the shape of the components
};

} // end of namespace mln::trait


namespace morpho
{

namespace attribute
{

/// inertia accumulator class.
///
/// The parameter \p I is the image type on which the accumulator
/// of pixels is built.
template <typename I>
struct inertia
: public mln::accu::internal::base< float, inertia<I> >
{
	typedef mln_psite(I) argument;

	inertia();

	/// Manipulators.
	/// \{
	void init();

	void take(const mln_psite(I)& p);
	void take(const inertia<I>& other);

	void take_as_init_(const mln_psite(I)& p);
	/// \}

	/// Get the value of the accumulator.
	float to_result() const;

	/// Check whether this accu is able to return a result.
	/// Always true here.
	bool is_valid() const;

//	/// Give the bounding box of the component.
//	box<mln_psite(I)> inertia() const;

protected:
	typename mln::accu::shape::inertia2d<mln_psite(I)> inertia_;
};



# ifndef MLN_INCLUDE_ONLY

template <typename I>
inline
inertia<I>::inertia()
{
	init();
}

template <typename I>
inline
void
inertia<I>::init()
{
	inertia_.init();
}

template <typename I>
inline
void
inertia<I>::take(const mln_psite(I)& p)
{
	if (! is_valid())
	{
		take_as_init_(p);
		return;
	}

	inertia_.take(p);
//	volume_.take(v);
//	height_.take(v);
}

template <typename I>
inline
void
inertia<I>::take(const inertia<I>& other)
{
	mln_invariant(is_valid());
	inertia_.take(other.inertia_);
//	volume_.take(other.volume_);
//	height_.take(other.height_);
}

template <typename I>
inline
void
inertia<I>::take_as_init_(const mln_psite(I)& p)
{
	inertia_.take_as_init_(p);
}

template <typename I>
inline
float
inertia<I>::to_result() const
{
	return (inertia_.to_result());
}

//template <typename I>
//inline
//unsigned
//inertia<I>::area() const
//{
//	return volume_.area();
//}
//
//template <typename I>
//inline
//unsigned
//sharpness<I>::volume() const
//{
//	return volume_.to_result();
//}
//
//template <typename I>
//inline
//unsigned
//inertia<I>::height() const
//{
//	return height_.to_result();
//}

template <typename I>
inline
bool
inertia<I>::is_valid() const
{
	return inertia_.is_valid();
}

# endif // ! MLN_INCLUDE_ONLY

} // end of namespace mln::morpho::attribute

} // end of namespace mln::morpho

} // end of namespace mln
#endif // ! MLN_MORPHO_ATTRIBUTE_INERTIA_HH

