/*
 * asf.hh
 *
 *  Created on: May 19, 2010
 *      Author: mauro
 */

#ifndef MLN_MORPHO_TREE_ASF_HH
#define MLN_MORPHO_TREE_ASF_HH

#include <mln/morpho/tree/tree.hh>
#include <mln/morpho/tree/filter/sub.hh>
#include <mln/morpho/tree/filter/min.hh>
#include <mln/estim/all.hh>
#include <mln/morpho/tree/propagate_representative.hh>
#include <mln/morpho/tree/compute_attribute_image.hh>

#include <mln/debug/println.hh>

namespace mln{

namespace morpho{

namespace tree{
// Given an attribute and a value of the attribute, compute the filtering as alternate sequential processing of the input data
// performed from the first value and progressively to the selected value.
// Perform a sequence of thinning-thickening on the input data.


template <typename F, typename A, typename N>
mln_concrete(F)
thin_thick(const Image<F>& in_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_);

template <typename F, typename A, typename N>
mln_concrete(F)
thick_thin(const Image<F>& in_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_);
//void thin_thick(const F& in_, F& out_, const N& nbh_, const A& a_, const mln_result(A)& lambda_);

# ifndef MLN_INCLUDE_ONLY
// Take_as_init specialization

namespace internal{

// perform an attribute thinning of the input image (by building a max-tree) with the lowest value of the attribute.
// If the predicate with the first value is satisfied then the input image is returned.
// The filtered image is returned in out_.
// FIXME: Specify if the evaluation of the predicate is done as > or <.
// FIXME: filtering rule should be specified also by the extern.
template <typename F, typename A, typename N>
bool
thin_first_value(const Image<F>& in_, Image<F>& out_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_){

	const F& in  = exact(in_);
	F& out = exact(out_);
	const N& nbh  = exact(nbh_);
	const A& a  = exact(a_);
	const mln_result(A)& lambda = exact(lambda_);

//	std::cout << "thin_first_value - internal\n";
//	  debug::println("in-internal = ", in);
	typedef p_array< mln_site(F) > S;
	typedef morpho::tree::data<F,S> tree_t;

	// compute the tree on the data
	S sorted_sites = data::sort_psites_increasing(in);	// max-tree
	tree_t tree(in, sorted_sites, nbh);

//	debug::println( "parent  = ", tree.parent_image() | tree.domain() );
//	debug::println( "on node = ", tree.parent_image() | tree.nodes()  );
//	debug::println( "on leaves = ", tree.parent_image() | tree.leaves()  );
//	debug::println( "children image = ", tree.children_image());

	typedef mln_ch_value(typename tree_t::function, mln_result(A)) attr_img;
	attr_img attr = morpho::tree::compute_attribute_image(a, tree);	// compute attribute
//	debug::println("attr_img = ", attr);
//	A a = morpho::tree::compute_attribute_image(a_, tree);	// compute attribute

	mln_value(attr_img) min_lambda;
	mln_value(attr_img) max_lambda;	// this is actually not used, is there a routine for computing only the min?

	mln::estim::min_max(attr, min_lambda, max_lambda);

//	std::cout << "min_lambda: " << min_lambda << std::endl;
//	std::cout << "max_lambda: " << max_lambda << std::endl;

	if(!(lambda > min_lambda))
			return false;

	// Redefinition of the predicate
    __typeof__(((pw::value(attr)) > pw::cst(min_lambda))) predicate = ((pw::value(attr)) > pw::cst(min_lambda));
    typedef __typeof__(((pw::value(attr)) > pw::cst(min_lambda))) predicate_t;
//	mln_VAR(predicate, pw::value(attr) > pw::cst(min_lambda));

//	initialize(output, in);
	morpho::tree::filter::sub(tree, out, predicate);
//	debug::println("out-filter = ", out);
	morpho::tree::propagate_representative(tree, out);
//	debug::println("out-internal = ", out);

	return true;
}

// perform an attribute thinning of the input image (by building a max-tree) with the lowest value of the attribute.
// If the predicate with the first value is satisfied then the input image is returned.
// The filtered image is returned in out_.
// FIXME: Specify if the evaluation of the predicate is done as > or <.
// FIXME: filtering rule should be specified also by the extern.
template <typename F, typename A, typename N>
bool
thick_first_value(const Image<F>& in_, Image<F>& out_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_){

	const F& in  = exact(in_);
	F& out = exact(out_);
	const N& nbh  = exact(nbh_);
	const A& a  = exact(a_);
	const mln_result(A)& lambda = exact(lambda_);

//	std::cout << "thick_first_value - internal\n";
//	  debug::println("in-internal = ", in);
	typedef p_array< mln_site(F) > S;
	typedef morpho::tree::data<F,S> tree_t;

	// compute the tree on the data
	S sorted_sites = data::sort_psites_decreasing(in);	// max-tree
	tree_t tree(in, sorted_sites, nbh);

//	debug::println( "parent  = ", tree.parent_image() | tree.domain() );
//	debug::println( "on node = ", tree.parent_image() | tree.nodes()  );
//	debug::println( "on leaves = ", tree.parent_image() | tree.leaves()  );
//	debug::println( "children image = ", tree.children_image());

	typedef mln_ch_value(typename tree_t::function, mln_result(A)) attr_img;
	attr_img attr = morpho::tree::compute_attribute_image(a, tree);	// compute attribute
//	debug::println("attr_img = ", attr);
//	A a = morpho::tree::compute_attribute_image(a_, tree);	// compute attribute

	mln_value(attr_img) min_lambda;
	mln_value(attr_img) max_lambda;	// this is actually not used, is there a routine for computing only the min?

	mln::estim::min_max(attr, min_lambda, max_lambda);

//	std::cout << "min_lambda: " << min_lambda << std::endl;
//	std::cout << "max_lambda: " << max_lambda << std::endl;

	if(!(lambda > min_lambda))
			return false;

	// Redefinition of the predicate
    __typeof__(((pw::value(attr)) > pw::cst(min_lambda))) predicate = ((pw::value(attr)) > pw::cst(min_lambda));
    typedef __typeof__(((pw::value(attr)) > pw::cst(min_lambda))) predicate_t;
//	mln_VAR(predicate, pw::value(attr) > pw::cst(min_lambda));

//	initialize(output, in);
	morpho::tree::filter::sub(tree, out, predicate);
//	debug::println("out-filter = ", out);
	morpho::tree::propagate_representative(tree, out);
//	debug::println("out-internal = ", out);

	return true;
}

template <typename F, typename A, typename N>
mln_concrete(F)
thin_thick(const Image<F>& in_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_){
//		std::cout << "thin_thick - internal\n";
	bool thin_process = true;
	bool thick_process = true;

	mln_concrete(F) output = duplicate(in_);
	mln_concrete(F) output1 = duplicate(in_);

	while (thin_process && thick_process){
		thin_process = thin_first_value(output, output1, nbh_, a_, lambda_);
		output = output1;
		thick_process = thick_first_value(output1, output, nbh_, a_, lambda_);
	}
	return output;
}

template <typename F, typename A, typename N>
mln_concrete(F)
thick_thin(const Image<F>& in_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_){
//		std::cout << "thin_thick - internal\n";
	bool thin_process = true;
	bool thick_process = true;

	mln_concrete(F) output = duplicate(in_);
	mln_concrete(F) output1 = duplicate(in_);

	while (thin_process && thick_process){
		thick_process = thick_first_value(output, output1, nbh_, a_, lambda_);
		output = output1;
		thin_process = thin_first_value(output1, output, nbh_, a_, lambda_);
	}
	return output;
}

}	// end of namespace mln::morpho::tree::internal


# endif // ! MLN_INCLUDE_ONLY

// Facade.
template <typename F, typename A, typename N>
inline
mln_concrete(F)
thin_thick(const Image<F>& in_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_){
	trace::entering("morpho::tree::thin_thick");
	mln_precondition(exact(in_).is_valid());

	mln_concrete(F) output = internal::thin_thick(in_, nbh_, a_, lambda_);
//	std::cout << "thin_thick - facade\n";

	trace::exiting("morpho::tree::thin_thick");

	return output;
}

template <typename F, typename A, typename N>
inline
mln_concrete(F)
thick_thin(const Image<F>& in_, const Neighborhood<N>& nbh_, const Accumulator<A>& a_, const mln_result(A)& lambda_){
	trace::entering("morpho::tree::thick_thin");
	mln_precondition(exact(in_).is_valid());

	mln_concrete(F) output = internal::thick_thin(in_, nbh_, a_, lambda_);
//	std::cout << "thick_thin - facade\n";

	trace::exiting("morpho::tree::thick_thin");

	return output;
}


}  // end of namespace mln::morpho::tree

}  // end of namespace mln::morpho

}  // end of namespace mln


#endif /* MLN_MORPHO_TREE_ASF_HH */
