/*
 * tree.hh
 *
 *  Created on: May 10, 2010
 *      Author: mauro
 */

#ifndef TREE_HH_
#define TREE_HH_

# include <mln/morpho/tree/data.hh>

namespace mln
{

namespace morpho
{

namespace tree
{

// Forward declarations.


// Return the sites belonging to the same connected component of the site p.
template <typename T>
p_array<mln_psite(T)> get_component_sites(const T& t, const mln_psite(T)& p);
// Same as the one above. Useful for recursive calls.
template <typename T>
void get_component_sites(const T& t, const mln_psite(T)& p, p_array<mln_psite(T)>& parr);

// Return the sites belonging to the peak of the site p (union of the component of p and of the components children of the node of p).
template <typename T>
p_array<mln_psite(T)> get_peak_sites(const T& t, const mln_psite(T)& p);
// Same as the one above. Useful for recursive calls.
template <typename T>
void get_peak_sites(const T& t, const mln_psite(T)& p, p_array<mln_psite(T)>& parr);

# ifndef MLN_INCLUDE_ONLY

template <typename T>
p_array<mln_psite(T)> get_component_sites(const T& t, const mln_psite(T)& p){

	p_array<mln_psite(T)> component_sites;
	get_component_sites<T>(t, p, component_sites);
	return (component_sites);
}

template <typename T>
void get_component_sites(const T& t, const mln_psite(T)& p, p_array<mln_psite(T)>& parr){

	mln_psite(T) component_root;
	if (!t.is_a_node(p))
		component_root = t.parent(p);
	else
		component_root = p;

	parr.insert(component_root);

	dn_site_piter<T> s(t);

	for_all(s){
		if (! t.is_a_node(s) && t.parent(s)==component_root)
			parr.insert(s);
	}
}

template <typename T>
p_array<mln_psite(T)> get_peak_sites(const T& t, const mln_psite(T)& p){

	p_array<mln_psite(T)> peak_sites;
	get_peak_sites<T>(t, p, peak_sites);
	return (peak_sites);
}

template <typename T>
void get_peak_sites(const T& t, const mln_psite(T)& p, p_array<mln_psite(T)>& parr){

	p_array<mln_psite(T)> component_sites = get_component_sites<T>(t, p);
	parr.append(component_sites);

	// Recursion on the children
	p_array<mln_psite(T)> children;
	if (!t.is_a_node(p))
		children = t.children(t.parent(p));
	else
		children = t.children(p);

	mln_piter(p_array<mln_psite(T)>) c(children);
	for_all(c){
		get_peak_sites<T>(t, c, parr);
	}
}

# endif // ! MLN_INCLUDE_ONLY


} // end of namespace mln::morpho::tree

} // end of namespace mln::morpho

} // end of namespace mln

#endif /* TREE_HH_ */
