/*
 * utility.hh
 *
 *  Created on: May 6, 2010
 *      Author: mauro
 */

#ifndef UTILITY_HH_
#define UTILITY_HH_

#include <mln/data/fill.hh>

namespace mln{

namespace data{


template <typename I, typename D>
void fill_random(Image<I>& ima, const D& data);


# ifndef MLN_INCLUDE_ONLY

namespace internal{

// tests

template <typename I, typename D>
inline
void fill_random_tests(Image<I>& ima, const D&)
{
	// Avoid a warning about an undefined variable when NDEBUG
	// is not defined.
	(void) ima;

	mlc_is(mln_trait_image_value_io(I), trait::image::value_io::read_write)::check();
	mln_precondition(exact(ima).is_valid());
	// FIXME: check for ambiguities...
}

// dispatch

template <typename I, typename D>
void fill_random_dispatch(Image<I>& ima, const D& data)
{
	fill_dispatch_overload(exact(ima), exact(data));
}

// dispatch_overload

template <typename I>
void fill_random_dispatch_overload(I& ima_, const mln_value(I)& v){


	typedef boost::minstd_rand base_generator_type;

		//	boost::mt19937 rng;

		//	base_generator_type generator(42u);
		//	base_generator_type generator(static_cast<unsigned int>(std::time(0)));
		base_generator_type generator(static_cast<unsigned int>(rand()));

		if (boost::is_floating_point<T>::value){
			boost::uniform_real<> uni_dist_f(min, max);
			boost::variate_generator<base_generator_type&, boost::uniform_real<> > uni_f(generator, uni_dist_f);
			for (unsigned long i=0; i<totalsize(); i++)
				_data[i] = uni_f();
		}
		else{
			boost::uniform_int<> uni_dist_i(min, max);
			//		boost::variate_generator<boost::mt19937&, boost::uniform_int<> > uni_i(rng, uni_dist_i);
			boost::variate_generator<base_generator_type&, boost::uniform_int<> > uni_i(generator, uni_dist_i);
			for (unsigned long i=0; i<totalsize(); i++)
				_data[i] = uni_i();
		}



	  I& ima = exact(ima_);

	  internal::fill_tests(ima, val);
	  mlc_is(mln_trait_image_pw_io(I),
			  trait::image::pw_io::read_write)::check();

	  mln_value(I) v = static_cast<mln_value(I)>(exact(val));

	  // Loop on all the sites of the image domain
	  mln_piter(I) p(ima.domain());
	  for_all(p){
		  val = ;
		  mln_value(I) v = static_cast<mln_value(I)>(exact(val));
		  ima(p) = v;
	  }
}

template <typename I, typename J>
void fill_dispatch_overload(I& ima, const Image<J>& data)
{
	mln::data::fill_with_image(ima, data);
}

template <typename I, typename F>
void fill_dispatch_overload(I& ima, const Function<F>& f)
{
	mlc_converts_to(mln_result(F), mln_value(I))::check();
	mln::data::fill_with_image(ima,
			exact(f) | ima.domain());
}

template <typename I, typename R, typename A>
void fill_dispatch_overload(I& ima, R (*f)(A))
{
	mlc_converts_to(R, mln_value(I))::check();
	mln::data::fill_with_image(ima,
			convert::to_fun(f) | ima.domain());
}

template <typename I, typename V, unsigned N>
void fill_dispatch_overload(I& ima, V (&arr)[N])
{
	mlc_converts_to(V, mln_value(I))::check();
	mln_precondition(N == ima.nsites());
	mln_fwd_piter(I) p(ima.domain());
	unsigned i = 0;
	for_all(p)
	ima(p) = arr[i++];
}

} // end of namespace mln::data::internal


// Facade.

template <typename I, typename D>
inline
void fill_random(Image<I>& ima, const D& data)
{
	trace::entering("data::fill_random");

	internal::fill_tests(ima, data);
	internal::fill_dispatch(ima, data);

	trace::exiting("data::fill_random");
}


# endif // ! MLN_INCLUDE_ONLY
}
}

#endif /* UTILITY_HH_ */
